//===================================================================================
// Microsoft patterns & practices
// Composite Application Guidance for Windows Presentation Foundation and Silverlight
//===================================================================================
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===================================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===================================================================================
using System;
using System.ComponentModel.Composition;
using System.Linq;
using IdeaBlade.EntityModel;
using Microsoft.Practices.Prism.Logging;
using Microsoft.Practices.Prism.MefExtensions.Modularity;
using Microsoft.Practices.Prism.Modularity;
using ModelA;
using ModuleTracking;

namespace ModuleA
{
    /// <summary>
    /// A module for the quickstart.
    /// </summary>
    [ModuleExport(typeof(ModuleA), DependsOnModuleNames = new string[] {"ModuleD"})]
    public class ModuleA : IModule
    {
        private readonly ILoggerFacade logger;
        private readonly IModuleTracker moduleTracker;

        /// <summary>
        /// Initializes a new instance of the <see cref="ModuleA"/> class.
        /// </summary>
        /// <param name="logger">The logger.</param>
        /// <param name="moduleTracker">The module tracker.</param>
        [ImportingConstructor]
        public ModuleA(ILoggerFacade logger, IModuleTracker moduleTracker)
        {
            if (logger == null)
            {
                throw new ArgumentNullException("logger");
            }

            if (moduleTracker == null)
            {
                throw new ArgumentNullException("moduleTracker");
            }

            this.logger = logger;
            this.moduleTracker = moduleTracker;
            this.moduleTracker.RecordModuleConstructed(WellKnownModuleNames.ModuleA);
        }

        /// <summary>
        /// Notifies the module that it has be initialized.
        /// </summary>
        public void Initialize()
        {
            this.logger.Log("ModuleA demonstrates logging during Initialize().", Category.Info, Priority.Medium);
            this.moduleTracker.RecordModuleInitialized(WellKnownModuleNames.ModuleA);
            InitializeDevForceComposition();
            RunQuery();
        }

        private void InitializeDevForceComposition() 
        {
          string xapName = string.Format("{0}.xap", WellKnownModuleNames.ModuleA);
          var uri = new Uri(xapName, UriKind.Relative);
          IdeaBlade.Core.Composition.CompositionHost.Add(uri);
        }

        private void RunQuery() 
        {
          var mgr = new NorthwindIBEntities();
          var op = mgr.Customers.ExecuteAsync<Customer>();
          op.Completed += (o, e) => {
            if (e.HasError) {
              this.moduleTracker.RecordEntitiesLoaded(WellKnownModuleNames.ModuleA, typeof(Customer), 0, e.Error);
              e.MarkErrorAsHandled();
            } else {
              this.moduleTracker.RecordEntitiesLoaded(WellKnownModuleNames.ModuleA, typeof(Customer), e.Results.Count(), null);
            }
          };
        }
    }
}
