//===================================================================================
// Microsoft patterns & practices
// Composite Application Guidance for Windows Presentation Foundation and Silverlight
//===================================================================================
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===================================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===================================================================================
using System;
using System.ComponentModel.Composition;
using System.Linq;
using IdeaBlade.EntityModel;
using Microsoft.Practices.Prism.MefExtensions.Modularity;
using Microsoft.Practices.Prism.Modularity;
using ModelF;
using ModuleTracking;

namespace ModuleF
{
    /// <summary>
    /// A module for the quickstart.
    /// </summary>
    [ModuleExport(typeof(ModuleF), InitializationMode = InitializationMode.OnDemand)]
    public class ModuleF : IModule
    {
        private readonly IModuleTracker moduleTracker;

        /// <summary>
        /// Initializes a new instance of the <see cref="ModuleF"/> class.
        /// </summary>
        /// <param name="moduleTracker">The module tracker.</param>
        [ImportingConstructor]
        public ModuleF(IModuleTracker moduleTracker)
        {
            if (moduleTracker == null)
            {
                throw new ArgumentNullException("moduleTracker");
            }

            this.moduleTracker = moduleTracker;
            this.moduleTracker.RecordModuleConstructed(WellKnownModuleNames.ModuleF);
        }

        /// <summary>
        /// Notifies the module that it has be initialized.
        /// </summary>
        public void Initialize()
        {
            this.moduleTracker.RecordModuleInitialized(WellKnownModuleNames.ModuleF);
            InitializeDevForceComposition();
        }

        private void InitializeDevForceComposition() {
          // If there's a lag in recomposition (as there is with Module F), we can listen
          // to the DevForce CompositionHost before issuing any queries.  Recomposition must
          // be complete before we can successfully use the EntityManager.
          IdeaBlade.Core.Composition.CompositionHost.Recomposed += CompositionHost_Recomposed;
          string xapName = string.Format("{0}.xap", WellKnownModuleNames.ModuleF);
          var uri = new Uri(xapName, UriKind.Relative);
          IdeaBlade.Core.Composition.CompositionHost.Add(uri);
        }

        private void CompositionHost_Recomposed(object sender, EventArgs e) {
          IdeaBlade.Core.Composition.CompositionHost.Recomposed -= CompositionHost_Recomposed;
          RunQuery();
        }

        private void RunQuery() {
          var mgr = new NorthwindIBEntities();
          var op = mgr.Orders.ExecuteAsync<Order>();
          op.Completed += (o, e) => {
            if (e.HasError) {
              this.moduleTracker.RecordEntitiesLoaded(WellKnownModuleNames.ModuleF, typeof(Order), 0, e.Error);
              e.MarkErrorAsHandled();
            } else {
              this.moduleTracker.RecordEntitiesLoaded(WellKnownModuleNames.ModuleF, typeof(Order), e.Results.Count(), null);
            }
          };
        }
    }
}
